#include <stdio.h>
#include <stdlib.h>

#include <clutter/clutter.h>
#include <tidy/tidy.h>

ClutterTimeline*
boing (const char *text)
{
  ClutterActor     *stage, *label;
  ClutterTimeline  *timeline;
  ClutterAlpha     *alpha;
  ClutterBehaviour *bounce;

  static            gint y = 0;

  stage = clutter_stage_get_default ();

  label = clutter_label_new_with_text ("Sans Bold 48", text);

  clutter_actor_set_position (label, 
                              (clutter_actor_get_width (stage)
                                 - clutter_actor_get_width (label))/2, 
                              y);

  y += ((3 *clutter_actor_get_height (label))/2);

  /* FIXME: behaviour should set this on apply.. */
  clutter_actor_set_opacity (label, 0);

  /* so we scale from center */
  clutter_actor_move_anchor_point_from_gravity (label,
                                                CLUTTER_GRAVITY_CENTER);
  
  clutter_container_add_actor (CLUTTER_CONTAINER (stage), label);

  timeline = clutter_timeline_new_for_duration (250); 

  alpha = clutter_alpha_new_full (timeline,
                                  CLUTTER_ALPHA_SINE_INC,
                                  NULL, NULL);

  /* Create a behaviour for that alpha */
  bounce = tidy_behaviour_bounce_new (alpha);

  /* Apply it to our actor */
  clutter_behaviour_apply (bounce, label);

  return timeline;
}

int
main (int argc, char *argv[])
{
  ClutterActor  *stage;
  ClutterScore  *score;
  ClutterTimeline *t, *tt;

  ClutterColor   stage_color = { 0xff, 0xff, 0xff, 0xff };

  clutter_init (&argc, &argv);

  stage = clutter_stage_get_default ();

  clutter_stage_set_color (CLUTTER_STAGE (stage),
			   &stage_color);

  score = clutter_score_new ();

  t = boing ("More Bounce");
  clutter_score_append (score, NULL, t);
  tt = boing ("To");
  clutter_score_append (score, t, tt);
  t = boing ("The");
  clutter_score_append (score, tt, t);
  tt = boing ("Ounce.");
  clutter_score_append (score, t, tt);

  clutter_actor_show_all (stage);

  clutter_score_start (score);

  clutter_main ();

  return EXIT_SUCCESS;
}
