#include <stdio.h>
#include <stdlib.h>

#include <clutter/clutter.h>
#include <tidy/tidy.h>

static ClutterActor *stage = NULL;

static gboolean
button_release (ClutterActor       *actor,
                ClutterButtonEvent *event,
                gpointer            user_data)
{
  g_debug ("Button release");

  return FALSE;
}

static gboolean
button_press (ClutterActor       *actor,
              ClutterButtonEvent *event,
              gpointer            user_data)
{
  g_debug ("Button press");

  return FALSE;
}

static void
button_clicked (TidyButton *button,
                gpointer    data)
{
  g_print ("[*] Button clicked (text: `%s')!\n",
           tidy_button_get_label (button));
}

int
main (int argc, char *argv[])
{
  ClutterActor *bg, *frame, *button;
  const gchar *text;
  ClutterColor stage_color =  { 0xff, 0xff, 0xff, 0xff };
  TidyPadding padding = { 0, };
  GError *error;

  clutter_init (&argc, &argv);

  text = argv[1];
  if (!text || text[0] == '\0')
    text = "<b>Hello</b>, <i>world</i>!";

  stage = clutter_stage_get_default ();
  clutter_stage_set_color (CLUTTER_STAGE (stage), &stage_color);

  padding.top = padding.bottom = 0; 
  padding.left = padding.right = 0;

  button = tidy_button_new_with_label (text);
  tidy_actor_set_padding (TIDY_ACTOR (button), &padding);
  tidy_actor_set_alignment (TIDY_ACTOR (button), 1.0, 1.0);
  clutter_container_add_actor (CLUTTER_CONTAINER (stage), button);

  clutter_actor_set_position (button, 100, 100);
  clutter_actor_set_size (button, 200, 50);
  g_signal_connect (button, "clicked",
                    G_CALLBACK (button_clicked),
                    NULL);
  g_signal_connect (button, "button-press-event",
                    G_CALLBACK (button_press),
                    NULL);
  g_signal_connect (button, "button-release-event",
                    G_CALLBACK (button_release),
                    NULL);

  clutter_actor_show (button);

  error = NULL;
  bg = clutter_texture_new_from_file ("frame.png", &error);
  if (error)
    g_error ("Unable to load pixbuf: %s", error->message);

  frame = tidy_texture_frame_new (CLUTTER_TEXTURE (bg), 20, 20, 20, 20);

  padding.top = padding.bottom = CLUTTER_UNITS_FROM_DEVICE (20);
  padding.left = padding.right = CLUTTER_UNITS_FROM_DEVICE (20);

  button = tidy_button_new_with_label (text);
  tidy_actor_set_padding (TIDY_ACTOR (button), &padding);
  tidy_frame_set_texture (TIDY_FRAME (button), frame);
  clutter_container_add_actor (CLUTTER_CONTAINER (stage), button);

  clutter_actor_set_position (button, 100, 200);
  g_signal_connect (button, "clicked",
                    G_CALLBACK (button_clicked),
                    NULL);

  clutter_actor_show (button);

  clutter_actor_show (stage);

  clutter_main ();

  g_object_unref (bg);

  return EXIT_SUCCESS;
}
