#include <clutter/clutter.h>
#include <tidy/tidy.h>

#include <stdlib.h>
#include <glib.h>

#define N_ROWS  5
#define ENABLE_TEXT     "Enable me!   "
#define DISABLE_TEXT    "Disable me!  "

static struct
{
  ClutterActor *label;
  ClutterActor *toggle;
} controls[N_ROWS];

static void
on_toggle_toggled (TidyFingerToggle *toggle)
{
  gboolean is_active = tidy_finger_toggle_get_active (toggle);
  const gchar *label_text;
  guint i;

  if (!is_active)
    label_text = ENABLE_TEXT;
  else
    label_text = DISABLE_TEXT;

  for (i = 0; i < N_ROWS; i++)
    {
      ClutterActor *actor = controls[i].toggle;
      ClutterLabel *label = CLUTTER_LABEL (controls[i].label);

      if (actor == CLUTTER_ACTOR (toggle))
        {
          clutter_label_set_text (label, label_text);
          return;
        }
    }
}

static ClutterActor *
create_toggle_group (gint index_)
{
  ClutterActor *group;
  ClutterActor *label;
  ClutterActor *texture;
  ClutterActor *toggle;
  GError *error = NULL;

  group = clutter_group_new ();

  label = clutter_label_new_with_text ("Sans 64px", ENABLE_TEXT);
  clutter_container_add_actor (CLUTTER_CONTAINER (group), label);
  clutter_actor_show (label);

  controls[index_].label = label;

  texture = clutter_texture_new_from_file ("toggle-bg.png", &error);
  if (error)
    g_error ("Unable to load `toggle-bg.png': %s\n", error->message);

  toggle = tidy_finger_toggle_new ();
  tidy_frame_set_texture (TIDY_FRAME (toggle), texture);
  clutter_container_add_actor (CLUTTER_CONTAINER (group), toggle);
  clutter_actor_set_xu (toggle, clutter_actor_get_widthu (label));
  clutter_actor_set_sizeu (toggle,
                           clutter_actor_get_widthu (texture),
                           clutter_actor_get_heightu (texture));
  clutter_actor_set_reactive (toggle, TRUE);
  g_signal_connect (toggle, "toggled",
                    G_CALLBACK (on_toggle_toggled),
                    NULL);
  clutter_actor_show (toggle);

  controls[index_].toggle = toggle;

  return group;
}

int
main (int   argc,
      char *argv[])
{
  ClutterActor *stage;
  ClutterColor stage_color = { 0x34, 0x39, 0x39, 0xff };
  ClutterUnit x_offset, y_offset;
  gint i;

  clutter_init (&argc, &argv);

  stage = clutter_stage_get_default ();
  clutter_stage_set_color (CLUTTER_STAGE (stage), &stage_color);
  clutter_actor_set_size (stage, 800, 480);

  x_offset = CLUTTER_UNITS_FROM_DEVICE (50);
  y_offset = CLUTTER_UNITS_FROM_DEVICE (50);
  for (i = 0; i < N_ROWS; i++)
    {
      ClutterActor *group = create_toggle_group (i);

      clutter_container_add_actor (CLUTTER_CONTAINER (stage), group);
      clutter_actor_set_positionu (group, x_offset, y_offset);
      clutter_actor_show_all (group);

      y_offset += clutter_actor_get_heightu (group);
      y_offset += CLUTTER_UNITS_FROM_DEVICE (10);
    }

  clutter_actor_show (stage);

  clutter_main ();

  return EXIT_SUCCESS;
}
