/* Try this text editor, it has issues but does work,
 * try ctrl+A, ctrl+C, ctrl+V, ctrl+X as well as selecting text with
 * mouse and keyboard, /Øyvind K
 */

#include <clutter/clutter.h>
#include <tidy/tidy-text.h>

#define FONT "Mono Bold 22px"

static gchar *clipboard = NULL;

static gchar *runes =
"ᚠᛇᚻ᛫ᛒᛦᚦ᛫ᚠᚱᚩᚠᚢᚱ᛫ᚠᛁᚱᚪ᛫ᚷᛖᚻᚹᛦᛚᚳᚢᛗ\n"
"ᛋᚳᛖᚪᛚ᛫ᚦᛖᚪᚻ᛫ᛗᚪᚾᚾᚪ᛫ᚷᛖᚻᚹᛦᛚᚳ᛫ᛗᛁᚳᛚᚢᚾ᛫ᚻᛦᛏ᛫ᛞᚫᛚᚪᚾ\n"
"ᚷᛁᚠ᛫ᚻᛖ᛫ᚹᛁᛚᛖ᛫ᚠᚩᚱ᛫ᛞᚱᛁᚻᛏᚾᛖ᛫ᛞᚩᛗᛖᛋ᛫ᚻᛚᛇᛏᚪᚾ᛬\n";


static gboolean
select_all (TidyText     *ttext,
            const gchar  *commandline,
            ClutterEvent *event)
{
  gint len;
  len = g_utf8_strlen (clutter_label_get_text (CLUTTER_LABEL (ttext)), -1);

  tidy_text_set_cursor_position (ttext, 0);
  tidy_text_set_selection_bound (ttext, len);

  return TRUE;
}

static gboolean
copy (TidyText     *ttext,
      const gchar  *commandline,
      ClutterEvent *event)
{
  if (clipboard)
    g_free (clipboard);
  clipboard = tidy_text_get_selection (ttext);
  return TRUE;
}

static gboolean
paste (TidyText     *ttext,
       const gchar  *commandline,
       ClutterEvent *event)
{
  const gchar *p;
  if (!clipboard)
    return TRUE;

  for (p=clipboard; *p!='\0'; p=g_utf8_next_char (p))
    {
      tidy_text_insert_unichar (ttext, g_utf8_get_char_validated (p, 3));
    }
  return TRUE;
}

static gboolean
cut (TidyText     *ttext,
     const gchar  *commandline,
     ClutterEvent *event)
{
  tidy_text_action (ttext, "copy", NULL);
  tidy_text_action (ttext, "truncate-selection", NULL);
  return TRUE;
}

static gboolean
pageup (TidyText     *ttext,
        const gchar  *commandline,
        ClutterEvent *event)
{
  gint i;
  for (i=0;i<10;i++)
    tidy_text_action (ttext, "move-up", event);
  return TRUE;
}

static gboolean
pagedown (TidyText     *ttext,
          const gchar  *commandline,
          ClutterEvent *event)
{
  gint i;
  for (i=0;i<10;i++)
    tidy_text_action (ttext, "move-down", event);
  return TRUE;
}

static void cursor_event (TidyText        *text,
                          ClutterGeometry *geometry)
{
  gint y;
 
  y = clutter_actor_get_y (CLUTTER_ACTOR (text));

  if (y + geometry->y < 50)
    {
      clutter_actor_set_y (CLUTTER_ACTOR (text), y + 100);
    }
  else if (y + geometry->y > 720)
    {
      clutter_actor_set_y (CLUTTER_ACTOR (text), y - 100);
    }

}

gint
main (gint    argc,
      gchar **argv)
{
  ClutterActor         *stage;
  ClutterActor         *text;
  ClutterColor          text_color       = {0x33, 0xff, 0x33, 0xff};
  ClutterColor          cursor_color     = {0xff, 0x33, 0x33, 0xff};
  ClutterColor          background_color = {0x00, 0x00, 0x00, 0xff};
  clutter_init (&argc, &argv);

  stage = clutter_stage_get_default ();
  clutter_stage_set_color (CLUTTER_STAGE (stage), &background_color);

  text = tidy_text_new_full (FONT, "·", &text_color);

  clutter_container_add (CLUTTER_CONTAINER (stage), text, NULL);
  clutter_actor_set_position (text, 40, 30);
  clutter_actor_set_width (text, 1024);
  clutter_label_set_line_wrap (CLUTTER_LABEL (text), TRUE);

  clutter_actor_set_reactive (text, TRUE);
  clutter_stage_set_key_focus (CLUTTER_STAGE (stage), text);

  tidy_text_set_editable (TIDY_TEXT (text), TRUE);
  tidy_text_set_selectable (TIDY_TEXT (text), TRUE);
  tidy_text_set_cursor_color (TIDY_TEXT (text), &cursor_color);

  tidy_text_add_action (TIDY_TEXT (text), "select-all", select_all);
  tidy_text_add_action (TIDY_TEXT (text), "copy",       copy);
  tidy_text_add_action (TIDY_TEXT (text), "paste",      paste);
  tidy_text_add_action (TIDY_TEXT (text), "cut",        cut);
  tidy_text_add_action (TIDY_TEXT (text), "pageup",     pageup);
  tidy_text_add_action (TIDY_TEXT (text), "pagedown",   pagedown);

  tidy_text_add_mapping (TIDY_TEXT (text),
        CLUTTER_a, CLUTTER_CONTROL_MASK, "select-all");
  tidy_text_add_mapping (TIDY_TEXT (text),
        CLUTTER_c, CLUTTER_CONTROL_MASK, "copy");
  tidy_text_add_mapping (TIDY_TEXT (text),
        CLUTTER_v, CLUTTER_CONTROL_MASK, "paste");
  tidy_text_add_mapping (TIDY_TEXT (text),
        CLUTTER_x, CLUTTER_CONTROL_MASK, "cut");
  tidy_text_add_mapping (TIDY_TEXT (text),
        CLUTTER_Page_Up, 0, "pageup");
  tidy_text_add_mapping (TIDY_TEXT (text),
        CLUTTER_Page_Down, 0, "pagedown");

  if (argv[1])
    {
      gchar                *utf8;
      g_file_get_contents (argv[1], &utf8, NULL, NULL);
      clutter_label_set_text (CLUTTER_LABEL (text), utf8);
    }
  else
    {
      clutter_label_set_text (CLUTTER_LABEL (text), runes);
    }

  g_signal_connect (text, "cursor-event", G_CALLBACK (cursor_event), NULL);

  clutter_actor_set_size (stage, 1024, 768);
  clutter_actor_show (stage);

  clutter_main ();
  return 0;
}
