#include <stdio.h>
#include <stdlib.h>

#include <clutter/clutter.h>
#include <tidy/tidy.h>

static int
nearest_pow2 (gint x)
{
  gint p = 1;
  
  while (p * p < x)
    ++p;
  
  return p;
}

int
main (int argc, char *argv[])
{
  gint           x, y, size;
  GDir          *dir;
  const gchar   *file;
  ClutterActor  *stage;
  ClutterColor   stage_color = { 0x0, 0x0, 0x0, 0xff };

  GList *files = NULL;
  GError *error = NULL;

  if (argc < 2)
    {
      printf ("Syntax: %s <path>\n", argv[0]);
      return EXIT_FAILURE;
    }

  /* Initialise threads so TidyProxyTexture loads textures asynchronously */
  g_thread_init (NULL);
  clutter_threads_init ();

  clutter_init (&argc, &argv);

  stage = clutter_stage_get_default ();
  clutter_stage_set_color (CLUTTER_STAGE (stage), &stage_color);
  clutter_actor_set_size (stage, 512, 512);

  if (!(dir = g_dir_open (argv[1], 0, &error)))
    {
      g_critical ("Error opening directory: %s", error->message);
      g_error_free (error);
      return 3;
    }
  
  while ((file = g_dir_read_name (dir)))
    {
      gchar *full_file;
      
      full_file = g_build_filename (argv[1], file, NULL);
      
      if (!g_file_test (full_file, G_FILE_TEST_IS_REGULAR))
        goto skip_file;
      
      if ((!g_str_has_suffix (full_file, ".png")) &&
          (!g_str_has_suffix (full_file, ".jpg")) &&
          (!g_str_has_suffix (full_file, ".PNG")) &&
          (!g_str_has_suffix (full_file, ".JPG")))
        goto skip_file;
      
      files = g_list_prepend (files, full_file);
      continue;

skip_file:
      g_free (full_file);
    }
  
  g_dir_close (dir);
  
  x = y = 0;
  size = MAX (16, 512 / nearest_pow2 (g_list_length (files)));

  while (files)
    {
      ClutterActor *tex;
      gchar *file = (gchar *)files->data;
      
      tex = g_object_new (TIDY_TYPE_PROXY_TEXTURE, NULL);
      clutter_actor_set_size (tex, size, size);
      g_object_set (tex, "path", file, NULL);
      clutter_actor_set_position (tex, x, y);
      clutter_container_add_actor (CLUTTER_CONTAINER (stage), tex);
      x += size;
      if (x > 512 - size)
        {
          x = 0;
          y += size;
        }
      
      g_free (file);
      files = g_list_delete_link (files, files);
    }

  
  clutter_actor_show_all (stage);

  clutter_main ();

  return EXIT_SUCCESS;
}
