#include <stdio.h>
#include <stdlib.h>

#include <clutter/clutter.h>
#include <tidy/tidy.h>

static inline void
usage (const gchar *message)
{
  g_print ("Usage: test-reflection <image>\n");

  if (!message)
    return;

  g_print ("Error: %s\n", message);
}

int
main (int argc, char *argv[])
{
  GError                *error = NULL;
  ClutterActor          *stage, *tex, *reflect;
  ClutterColor           stage_color = { 0x0, 0x0, 0x0, 0xff };
  gint                   x;
  ClutterEffectTemplate *tmpl;

  if (argc < 2)
    {
      usage (NULL);
      return EXIT_FAILURE;
    }

  clutter_init (&argc, &argv);

  stage = clutter_stage_get_default ();
  clutter_stage_set_color (CLUTTER_STAGE (stage), &stage_color);

  tex = clutter_texture_new_from_file (argv[1], &error);
  clutter_texture_set_filter_quality (CLUTTER_TEXTURE (tex),
                                      CLUTTER_TEXTURE_QUALITY_LOW);
  if (error)
    {
      usage (error->message);
      g_error_free (error);
      return EXIT_FAILURE;
    }

  reflect = tidy_texture_reflection_new (CLUTTER_TEXTURE (tex));
  clutter_actor_set_opacity (reflect, 100);

  x = (CLUTTER_STAGE_WIDTH () - clutter_actor_get_width (tex)) / 2;

  clutter_group_add (CLUTTER_GROUP (stage), tex);
  clutter_group_add (CLUTTER_GROUP (stage), reflect);
  clutter_actor_set_position (tex, x, 20);
  clutter_actor_set_position (reflect, x, clutter_actor_get_height (tex) + 20);

  /* clutter_actor_rotate_y (stage, 60.0, CLUTTER_STAGE_WIDTH()/2, 0); */

  clutter_actor_show_all (stage);

  tmpl =
    clutter_effect_template_new_for_duration (5000, CLUTTER_ALPHA_RAMP_INC);
  clutter_effect_fade (tmpl, tex, 0, NULL, NULL);
  clutter_effect_fade (tmpl, reflect, 0, NULL, NULL);

  clutter_main();

  g_object_unref (tmpl);

  return EXIT_SUCCESS;
}
